import { DatabaseSchema } from "./mysql-database-definition";
import Knex = require('knex');
import ModelBuilder from "./model-builder";
/**
 * This class serves as a data access object and allowes communication with the mysql server
 * It automatically strips away attributes that not match columns
 */
export default class TypedInserter {
    private knex;
    private dbModel;
    constructor(knex: Knex, dbModel?: DatabaseSchema);
    /**
     * init a database schema if none were provided at constructor
     */
    init(): Promise<ModelBuilder>;
    batchInsert<T>(tableName: string, rows: T[]): Promise<void>;
    batchInsertIgnore<T>(tableName: string, rows: T[]): Promise<void>;
    batchUpdate<T>(tableName: string, data: T[]): Promise<void>;
    /**
     * return true if object has its primary key among its items
     * @param tableName
     * @param data
     */
    hasPrimaryKey<T>(tableName: string, data: T): boolean;
    private insert<T>(tableName, data);
    /**
     * Insert an object, if it exist, then silentely ignore and continue
     * @param tableName
     * @param item
     */
    insertIgnore<T>(tableName: string, item: T): Promise<void>;
    insertOnDuplicateUpdate<T>(tableName: string, data: T): Promise<void>;
    /***
    * @param tableName
    * @param object
    * Replaces all tinyint columns with a boolean
    */
    boolfix<T>(tableName: string, object: T): T;
    /**
    Returns the same object with all date fields
    strings parsed into date objects
    */
    datefix<T>(tableName: string, object: T): T;
    /**
     * @param tableName
     * @param data
     * @param forceInsert
     * Insert an object that already has its primary key among its variables
     */
    insertWithPrimaryKey<T>(tableName: string, data: T, forceInsert?: boolean): Promise<T>;
    /**
     * returns true if the first primary key column is of type int
     * @param tableName
     */
    isIntPk(tableName: string): boolean;
    private getPkCols(tableName);
    /**
     * List the names of all primary keys in a table
     * @param tableName
     */
    private getPrimaryKeyNames(tableName);
    /**
     * Returns the name of the first primary key column
     * @param tableName
     */
    getPrimaryKey(tableName: string): string;
    /**
     * @param tableName
     * @param data
     * Return a copy of the object without any primary keys
     */
    removePrimaryKeys<T>(tableName: string, data: T): T;
    /**
     * Insert but if the object exists throw the other one away.
     * @param tableName
     * @param item
     */
    replaceInto<T>(tableName: string, item: T): Promise<void>;
    /**
     * Update or insert a data objet.
     * Update if object has a primary key already.
     * To insert an object with a primary key, use the other function
     * @param tableName
     * @param data
     */
    setData<T>(tableName: string, data: T): Promise<T>;
    /**
     * return a new object where with only attributes that have
     * a corresponding column for given table
     * @param table
     * @param data
     */
    private stripNoneBelonging<T>(tableName, data);
    forceUpdate<T>(tableName: string, data: T): Promise<T>;
    private update<T>(tableName, data);
}
